﻿
''' <summary>
''' Klasse zur Berechnung einer CRC8-Prüfsumme
''' </summary>
''' <remarks>Benutzt eine Look-Up-Tabelle. <para/>
'''    Polynom:   0x07  <para/>
'''    Startwert: 0xFF
''' </remarks>
Public Class CRC8GeneratorLookUp

#Region "==== Paket Methoden ====="

   ''' <summary>
   ''' Generiert die CRC8 für ein Datenpaket.
   ''' </summary>
   ''' <param name="data">Das Datenpaket, zu dem die CRC8-Prüfsumme berechent werden soll.</param>
   ''' <returns>Die berechente CRC8-Prüfsumme.</returns>
   ''' <remarks>Benutzt eine Look-Up-Tabelle.</remarks>
   Public Function genCRC(ByVal Data As Byte()) As Byte
      If (Data Is Nothing) Then
         Return crcSeed
      End If

      Dim crc = crcSeed
      Dim len = Data.Length
      Dim index As Integer = 0

      Do While (len > 0)
         crc = crc8Table(crc Xor Data(index))
         len -= 1
         index += 1
      Loop

      Dim tmp8 As Byte
      Dim testBit As Byte
      Dim bitMask8 As Byte

      tmp8 = 0
      testBit = 1
      bitMask8 = 1 << 7
      Do While bitMask8 <> 0
         If ((crc And bitMask8) <> 0) Then
            tmp8 = tmp8 Or testBit
         End If
         testBit <<= 1
         bitMask8 >>= 1
      Loop
      crc = CByte(tmp8 Xor &HFF)


      Return crc
   End Function
#End Region

#Region "==== Stream Methoden ====="

   ''' <summary>
   ''' Gibt ein Byte an den Datenstrom ab und berechnet dabei fortlaufend die CRC.
   ''' </summary>
   ''' <param name="b">Das weiter zu gebende Byte.</param>
   ''' <remarks>Benutzt eine Look-Up-Tabelle.<para/>
   ''' Zur Weitergabe wird die Methode <see cref="executor"/> benutzt.</remarks>
   Public Sub streamCRC(b As Byte)
      crc = crc8Table(crc Xor b)
      executor(b)
   End Sub

   ''' <summary>
   ''' Gibt die CRC an den Datenstrom weiter und initialisiert erneut die internen Variablen.
   ''' </summary>
   ''' <returns>Die weiter gegebene Checksumme.</returns>
   Public Function streamFin() As Byte
      Dim tmp8 As Byte
      Dim testBit As Byte
      Dim bitMask8 As Byte

      tmp8 = 0
      testBit = 1
      bitMask8 = 1 << 7
      Do While bitMask8 <> 0
         If ((crc And bitMask8) <> 0) Then
            tmp8 = tmp8 Or testBit
         End If
         testBit <<= 1
         bitMask8 >>= 1
      Loop
      crc = CByte(tmp8 Xor &HFF)

      executor(crc)

      Dim temp = crc
      crc = crcSeed
      Return temp
   End Function
#End Region


   ''' <summary>
   ''' Datentyp für eine Methode zur Weitergabe eines Bytes an den Datenstrom.
   ''' </summary>
   ''' <param name="b">Das weiter zu gebende Byte.</para
   Public Delegate Sub executorDelegate(b As Byte)

   ''' <summary>
   '''  Methode zur Weitergabe eines Bytes an den Datenstrom.
   ''' </summary>
   Public executor As executorDelegate = New executorDelegate(AddressOf dummyExcecutor)

#Region "==== Geschützter Bereich ====="

   ''' <summary>
   ''' Leere Funktion zur initialisierung der Variable 'executor'.
   ''' </summary>
   ''' <param name="b">Das weiter zu gebende Byte.</param>
   Protected Shared Sub dummyExcecutor(b As Byte)
   End Sub

   ''' <summary>
   ''' Startwert der CRC-Berechnung.
   ''' </summary>
   Private Shared ReadOnly crcSeed As Byte = &HFF

   ''' <summary>
   ''' Zwischenspeicher für die fortlaufende Berechnung.
   ''' </summary>
   Private crc As Byte = crcSeed

   ''' <summary>
   ''' Look-Up-Tabelle
   ''' </summary>
   Private Shared ReadOnly crc8Table As Byte() = New Byte() {
      &H0, &H91, &HE3, &H72, &H7, &H96, &HE4, &H75,
      &HE, &H9F, &HED, &H7C, &H9, &H98, &HEA, &H7B,
      &H1C, &H8D, &HFF, &H6E, &H1B, &H8A, &HF8, &H69,
      &H12, &H83, &HF1, &H60, &H15, &H84, &HF6, &H67,
      &H38, &HA9, &HDB, &H4A, &H3F, &HAE, &HDC, &H4D,
      &H36, &HA7, &HD5, &H44, &H31, &HA0, &HD2, &H43,
      &H24, &HB5, &HC7, &H56, &H23, &HB2, &HC0, &H51,
      &H2A, &HBB, &HC9, &H58, &H2D, &HBC, &HCE, &H5F,
      &H70, &HE1, &H93, &H2, &H77, &HE6, &H94, &H5,
      &H7E, &HEF, &H9D, &HC, &H79, &HE8, &H9A, &HB,
      &H6C, &HFD, &H8F, &H1E, &H6B, &HFA, &H88, &H19,
      &H62, &HF3, &H81, &H10, &H65, &HF4, &H86, &H17,
      &H48, &HD9, &HAB, &H3A, &H4F, &HDE, &HAC, &H3D,
      &H46, &HD7, &HA5, &H34, &H41, &HD0, &HA2, &H33,
      &H54, &HC5, &HB7, &H26, &H53, &HC2, &HB0, &H21,
      &H5A, &HCB, &HB9, &H28, &H5D, &HCC, &HBE, &H2F,
      &HE0, &H71, &H3, &H92, &HE7, &H76, &H4, &H95,
      &HEE, &H7F, &HD, &H9C, &HE9, &H78, &HA, &H9B,
      &HFC, &H6D, &H1F, &H8E, &HFB, &H6A, &H18, &H89,
      &HF2, &H63, &H11, &H80, &HF5, &H64, &H16, &H87,
      &HD8, &H49, &H3B, &HAA, &HDF, &H4E, &H3C, &HAD,
      &HD6, &H47, &H35, &HA4, &HD1, &H40, &H32, &HA3,
      &HC4, &H55, &H27, &HB6, &HC3, &H52, &H20, &HB1,
      &HCA, &H5B, &H29, &HB8, &HCD, &H5C, &H2E, &HBF,
      &H90, &H1, &H73, &HE2, &H97, &H6, &H74, &HE5,
      &H9E, &HF, &H7D, &HEC, &H99, &H8, &H7A, &HEB,
      &H8C, &H1D, &H6F, &HFE, &H8B, &H1A, &H68, &HF9,
      &H82, &H13, &H61, &HF0, &H85, &H14, &H66, &HF7,
      &HA8, &H39, &H4B, &HDA, &HAF, &H3E, &H4C, &HDD,
      &HA6, &H37, &H45, &HD4, &HA1, &H30, &H42, &HD3,
      &HB4, &H25, &H57, &HC6, &HB3, &H22, &H50, &HC1,
      &HBA, &H2B, &H59, &HC8, &HBD, &H2C, &H5E, &HCF
   }
#End Region

End Class
